@@ -43,14 +43,15 @@ class TrytonConfigParser(ConfigParser.RawConfigParser):
         ConfigParser.RawConfigParser.__init__(self)
         self.add_section('web')
         self.set('web', 'listen', 'localhost:8000')
-        self.set('web', 'root', '/var/www/localhost/tryton')
+        self.set('web', 'root', os.path.join(os.path.expanduser('~'), 'www'))
         self.add_section('database')
         self.set('database', 'uri',
             os.environ.get('TRYTOND_DATABASE_URI', 'sqlite://'))
-        self.set('database', 'path', '/var/lib/trytond')
+        self.set('database', 'path', os.path.join(
+                os.path.expanduser('~'), 'db'))
         self.set('database', 'list', 'True')
         self.set('database', 'retry', 5)
-        self.set('database', 'language', 'en_US')
+        self.set('database', 'language', 'en')
         self.add_section('cache')
         self.set('cache', 'model', 200)
         self.set('cache', 'record', 2000)
@@ -59,6 +60,7 @@ class TrytonConfigParser(ConfigParser.RawConfigParser):
         self.add_section('email')
         self.set('email', 'uri', 'smtp://localhost:25')
         self.add_section('session')
+        self.set('session', 'authentications', 'password')
         self.set('session', 'timeout', 600)
         self.add_section('report')
         self.set('report', 'unoconv',
@@ -68,8 +70,8 @@ class TrytonConfigParser(ConfigParser.RawConfigParser):
     def update_etc(self, configfile=os.environ.get('TRYTOND_CONFIG')):
         if not configfile:
             return
-        logger.info('using %s as configuration file', configfile)
-        self.read(configfile)
+        read_files = self.read(configfile)
+        logger.info('using %s as configuration files', ', '.join(read_files))
 
     def get(self, section, option, *args, **kwargs):
         default = kwargs.pop('default', None)
